<?php /* хелпер шаблона  */

    // =======================================================================
    /**
    *  Загрузка расширения хелпера (коммерческие модули), если есть
    */
    // =======================================================================

    $file = dirname(__FILE__) . '/mod-helper.php';
    if (file_exists($file)) {
        require_once($file);
    } else {
        class TemplateEmulatorMods {
            protected function setSmartyPlugins ( & $cms ) {
            }
        }
    }

    // =======================================================================
    /**
    *  Непосредственно хелпер
    */
    // =======================================================================

    class TemplateEmulator extends TemplateEmulatorMods {
        protected $cms = null;

        // ===================================================================
        /**
        *  Регистрация наших шаблонизационных функций
        */
        // ===================================================================

        protected function setSmartyPlugins ( & $cms ) {
            parent::setSmartyPlugins($cms);
            $cms->smarty->registerPlugin('function', 'version', array($this, 'version'));
            $cms->smarty->registerPlugin('function', 'versionYMD', array($this, 'versionYMD'));
            $cms->smarty->registerPlugin('function', 'header200', array($this, 'header200'));
            $cms->smarty->registerPlugin('function', 'metaDescription', array($this, 'metaDescription'));
            $cms->smarty->registerPlugin('function', 'metaKeywords', array($this, 'metaKeywords'));
            $cms->smarty->registerPlugin('function', 'siteName', array($this, 'siteName'));
            $cms->smarty->registerPlugin('function', 'configParam', array($this, 'configParam'));
            $cms->smarty->registerPlugin('function', 'configCommaList', array($this, 'configCommaList'));
            $cms->smarty->registerPlugin('function', 'phone', array($this, 'phone'));
            $cms->smarty->registerPlugin('function', 'counters', array($this, 'counters'));
            $cms->smarty->registerPlugin('function', 'titleSearch', array($this, 'titleSearch'));
            $cms->smarty->registerPlugin('function', 'pagination', array($this, 'pagination'));
            $cms->smarty->registerPlugin('function', 'centralTemplate', array($this, 'centralTemplate'));
            $cms->smarty->assignByRef('emulator', $this);
        }

        // ===================================================================
        /**
        *  Подготовка к отрисовке конкретного файла шаблона
        */
        // ===================================================================

        public function prepare ( $template, & $cms, & $body = null ) {
            $body = null;
            $name = strtolower($template->name);
            switch ($name) {
                case 'index.htm':
                case 'index.tpl':
                    $url = $this->cms->request->getServerAsSentence('REQUEST_URI');
                    $url = rtrim(preg_replace('![/\\\\]+!u', '/', $url), '/. ');
                    $url = explode('/', $url);
                    $url = trim(array_pop($url));
                    if (strtolower($url) == 'setup') {
                        if ($this->cms->request->isPosted()) {
                            if ($this->cms->config->demo) {
                                $this->cms->smarty->assign('error', 'В демо версии отклоняются попытки сохранить изменения в настройках шаблона.');
                            } else {
                            }
                        }
                    }
                    break;
            }
            return $template->name;
        }

        // ===================================================================
        /**
        *  Обработчики наших функций
        */
        // ===================================================================

        public function version ( $params = null, & $smarty = null ) {
            $value = IMPERA_CMS_CURRENT_MMVERSION;
            return $this->asAttribute($value);
        }

        public function versionYMD ( $params = null, & $smarty = null ) {
            $value = IMPERA_CMS_CURRENT_VERSION;
            return $this->asAttribute($value);
        }

        public function header200 ( $params = null, & $smarty = null ) {
            @ header('HTTP/1.1 200 OK', TRUE, 200);
            return '';
        }

        public function metaDescription ( $params = null, & $smarty = null ) {
            $value = $this->getVar('description', $smarty);
            $value = $this->mustbeString($value);
            return $this->asAttribute($value);
        }

        public function metaKeywords ( $params = null, & $smarty = null ) {
            $value = $this->getVar('keywords', $smarty);
            $value = $this->mustbeString($value);
            return $this->asAttribute($value);
        }

        public function siteName ( $params = null, & $smarty = null ) {
            $value = $this->cms->any->settings->getAsSentence('site_name');
            return $this->asAttribute($value);
        }

        public function configParam ( $params = null, & $smarty = null ) {
            $param = $this->paramAsString('name', $params);
            $value = $this->getConfig($param);
            if ($value == '') $value = $this->paramAsString('def', $params);
            return $value;
        }

        public function configCommaList ( $params = null, & $smarty = null ) {
            $param = $this->paramAsString('name', $params);
            $value = $this->getConfig($param);
            $items = explode(',', $value);
            $value = array();
            foreach ($items as $item) {
                $item = trim($item);
                if ($item != '') $value[] = $item;
            }
            if (empty($value)) $value = $this->paramAsArray('def', $params);
            return $this->assignVar($params, $value, $smarty);
        }

        public function phone ( $params = null, & $smarty = null ) {
            $param = 'phone' . $this->paramAsNatural('num', $params);
            $value = $this->getConfig($param);
            return $this->asAttribute($value);
        }

        public function counters ( $params = null, & $smarty = null ) {
            return $this->cms->any->settings->get('counters', '');
        }

        public function titleSearch ( $params = null, & $smarty = null ) {
            $value = $this->getVar('search_title', $smarty);
            $value = $this->mustbeString($value);
            return $this->asAttribute($value);
        }

        public function pagination ( $params = null, & $smarty = null ) {
            $value = $this->getVar('PagesNavigation', $smarty);
            return $this->mustbeString($value);
        }

        public function centralTemplate ( $params = null, & $smarty = null ) {
            $this->cms->quick_content = FALSE;
            return '';
        }

        // ===================================================================
        /**
        *  Функции объекта $emulator
        */
        // ===================================================================

        public function existsModule ( $file ) {
            $file = $this->cms->hdd->safeFilename($file);
            if (empty($file)) return FALSE;
            $ext = explode('.', $file);
            if (count($ext) < 2) return FALSE;
            $ext = array_pop($ext);
            $ext = $this->cms->text->lowerCase($ext);
            if (!in_array($ext, array('tpl', 'htm', 'html', 'xml', 'css', 'js', 'txt'))) return FALSE;
            return $this->cms->hdd->isReadableFile(dirname(__FILE__) . '/' . $file);
        }

        public function existsInStrings ( $value, $strings ) {
            if (is_array($strings)) {
                $value = $this->cms->text->lowerCase($value);
                foreach ($strings as $item) {
                    if ($this->cms->text->lowerCase($item) == $value) return TRUE;
                }
            }
            return FALSE;
        }

        // ===================================================================
        /**
        *  Системные методы
        */
        // ===================================================================

        public function __construct ( & $cms ) {
            $this->cms = & $cms;
            $this->setSmartyPlugins($cms);
        }

        private function getVar ( $varname, & $smarty ) {
            $value = $smarty->getTemplateVars($varname);
            if (is_null($value)) $value = $this->cms->smarty->getTemplateVars($varname);
            return $value;
        }

        private function getConfig ( $paramname ) {
            return isset($this->cms->config->$paramname) ? trim($this->cms->config->$paramname) : '';
        }

        private function paramAsString ( $paramname, & $params ) {
            return isset($params[$paramname]) ? trim($params[$paramname]) : '';
        }

        private function paramAsNatural ( $paramname, & $params ) {
            $value = isset($params[$paramname]) ? intval($params[$paramname]) : 1;
            return max(1, $value);
        }

        private function paramAsArray ( $paramname, & $params ) {
            return isset($params[$paramname]) && is_array($params[$paramname]) ? $params[$paramname] : array();
        }

        private function asAttribute ( & $value ) {
            return htmlspecialchars($value, ENT_QUOTES, 'UTF-8');
        }

        private function mustbeString ( & $value ) {
            return is_string($value) ? $value : '';
        }

        private function assignVar ( & $params, & $value, & $smarty, $defname = '' ) {
            if (!is_object($smarty)) return $value;
            $varname = !empty($params['var']) && is_string($params['var'])
                           ? trim($params['var'])
                           : (!empty($params['assign']) && is_string($params['assign'])
                               ? trim($params['assign'])
                               : (!empty($params['result']) && is_string($params['result'])
                                   ? trim($params['result'])
                                   : trim($defname)));
            if (empty($varname)) return $value;
            $smarty->assignByRef($varname, $value);
            if (isset($params['scope']) && $params['scope'] == 'global') {
                $this->cms->smarty->assignByRef($varname, $value);
            }
            return '';
        }
    }

    return;
?>